/*
 * ops.c - Higher-level toolpath operations
 *
 * Written 2010 by Werner Almesberger
 * Copyright 2010 Werner Almesberger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */


#include "path.h"
#include "ops.h"


static void tool_comp_1(struct path *path, int inside, int dog_bone)
{
	int left;
	struct path *new;

	left = path_tool_is_left(path);
	if (inside)
		new = path_offset(path, !left, path->notch);
	else
		new = path_offset(path, left, path->notch || dog_bone);
	path_replace(path, new);
}


void tool_comp_paths(struct path *paths, int dog_bone)
{
	struct path *leftmost, *path;

	/*
	 * We don't have an algorithm (yet) that can detect which paths are
	 * inside other paths. Therefore, we fake it by looking for the path
	 * that contains lowest x coordinate. This ought to be the outer
	 * boundary of the piece.
	 *
	 * Note that this heuristic falls apart when a job consists of
	 * multiple pieces. In this case, the #%outside hint can be used to
	 * explicitly tell cameo to treat the path as an outside edge.
	 */

	leftmost = path_find_leftmost(paths);
	for (path = paths; path; path = path->next)
		if (path != leftmost && !path->outside)
			tool_comp_1(path, 1, dog_bone);
	for (path = paths; path; path = path->next)
		if (path != leftmost && path->outside)
			tool_comp_1(path, 0, dog_bone);
	tool_comp_1(leftmost, 0, dog_bone);
}
